@@ -19,6 +19,7 @@
 #' @param schema An alternative schema to be accessed.  Used for testing purposes.
 #' @param print.query Should  the query used be printed?  Default is FALSE
 #' @param fetch.query If TRUE (the default) query is executed using dbFetch.  If FALSE, dbGetQuery is used.
+#' @param record_limit The number of records to download at once. Defaults to 10000
 #' @note Using fetch.query = TRUE provides better error handling, but fetch.query = FALSE results in a more useful (but uncatchable) error.
 #' @import RPostgreSQL
 #' @importFrom DBI dbDriver dbFetch
@@ -44,7 +45,8 @@
                       return.query = FALSE,
                       schema = NULL,
                       print.query = FALSE,
-                      fetch.query = TRUE){
+                      fetch.query = TRUE,
+                      record_limit = 10000 ){
   
   .is_char(query)
 
@@ -199,32 +201,121 @@
   drv <- dbDriver('PostgreSQL')
   # establish connection with database
   
+  if(return.query){
+    query <- gsub(pattern = "\n", replacement = "", query)
+    query <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl = TRUE)
+    return(query)
+  }
+  
+  
+# Try to connect
+
   con <- dbConnect(drv,
                    host = host,
                    dbname = dbname,
                    user = user,
                    password = password)
   
-  
-  if(return.query){
-    query <- gsub(pattern = "\n", replacement = "", query)
-    query <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl = TRUE)
-    dbDisconnect(con)
-    return(query)
+  if("error" %in% class(con)){
+    
+    message("\n There was an error connecting to the BIEN database.")
+    
+    rm(con)
+    
+    return(invisible(NULL))
+    
   }
+  
 
 
 
 # Execute the query
   
   if(fetch.query){
+
+    # Send the query
+    
+          suppressWarnings(
+            
+            
+            res <-  tryCatch(expr = dbSendQuery(conn = con,
+                                                statement = query),
+                            error = function(e){e}
+            )
+          )
+        
+        if("error" %in% class(res)){
+          
+          message("\n There was an error in the SQL query or the SQL query could not be sent (e.g., due to connection issues).")
+          
+          suppressWarnings(
+            
+            db_cleared <- tryCatch(expr = dbClearResult(res = res),
+                     error = function(e){e}
+            )
+          )   
+          
+          dbDisconnect(con)
+          
+          return(invisible(NULL))
+          
+        }
+    
+    
+    
+    
+    
+    
+    # Fetch the query    
+        # suppressWarnings(
+        #   
+        #   df <-  tryCatch(expr = dbFetch(res = res),
+        #                   error = function(e){e}
+        #                   )
+        #   )
+    
+    
+    
+    df <- NULL
+    batch <- 1
+    
+    while(!dbHasCompleted(res)){
+      
+      message("Getting page ", batch, " of records")
+      
+      df_x <-  tryCatch(expr = dbFetch(res = res,
+                                       n = record_limit),
+                        error = function(e){e}
+      )
+      
+      #check for errors
+      if("error" %in% class(df_x)){
+        
+        message("Error fetching page ", batch, " of records.")
+        dbClearResult(res)
+        return(invisible(NULL))
+      }
+      
+      df_x |>
+        rbind(df) -> df
+      
+      batch = batch+1
+      
+      
+    }
+    
+  
+    # Clear the query results
+        
+      suppressWarnings(
+        
+        db_cleared <- tryCatch(expr = dbClearResult(res = res),
+                        error = function(e){e}
+        )
+      )   
+  
     
-    suppressWarnings(
       
-      df <-  tryCatch(expr = dbFetch(res = dbSendQuery(conn = con,
-                                                       statement = query)),
-                      error = function(e){e}
-      ))
     
   }else{
     
@@ -252,7 +343,7 @@
   
   if("error" %in% class(df)){
     
-    message("\nThere was a problem with the query. This is most often due to internet connection issues, but may also be due other factors such as an outdated version of the package.")
+    message("\nThe query was properly executed, but there was a problem retreiving the query results. This may be due to internet connection issues.")
     
     return(invisible(NULL))
     
